const m = require('./pointer.js')

const mallocByteBuffer = len => {
    const ptr = m._malloc(len)
    const heapBytes = new Uint8Array(m.HEAPU8.buffer, ptr, len)
    return heapBytes
}

const mallocUInt32Buffer = len =>  {
    const ptr = m._malloc(len * 4)
    const heapBytes = new Uint32Array(m.HEAPU8.buffer, ptr, len)
    return heapBytes
}

const free  = nativeBuffer =>  {
    if (typeof nativeBuffer === 'number') {
        m._free(nativeBuffer)
    } else if (nativeBuffer && nativeBuffer.buffer) {
        nativeBuffer.buffer instanceof ArrayBuffer && m._free(nativeBuffer.byteOffset)
    }
}

// 交换数字
const swapNumbers = ({first = 0, second  = 0} = {}) => {
    var aPtr = m._malloc(4)
    var bPtr = m._malloc(4)
    m.setValue(aPtr, first, 'i32')
    m.setValue(bPtr, second, 'i32')
    m.ccall('swap', null, ['number', 'number'], [aPtr, bPtr])
    return {
        first: m.getValue(aPtr, 'i32'),
        second: m.getValue(bPtr, 'i32')
    }
}

// 数组数字加1
const plusArrays = (numbers = []) => {
    // 传递数组
    const length = numbers.length
    const inputBuffer = mallocUInt32Buffer(length)
    const outputBuffer = mallocUInt32Buffer(length)
    // 填充数据
    inputBuffer.set(numbers)
    m.ccall('addOne', null, ['number', 'number', 'number'], [inputBuffer.byteOffset, outputBuffer.byteOffset, length])
    const outputArray = new Int32Array(outputBuffer.buffer, outputBuffer.byteOffset, length)
    free(inputBuffer)
    free(outputBuffer)
    return Array.from(outputArray)
}

// 检测字符串中某个字符出现的次数
const countOccurrences = (words, target = '') => {
    // 传递字符串
    const countOccurrences = m.cwrap("countOccurrences", "number", ["number", "number", "number"]) 
    const array = words.split('').map(v => v.charCodeAt(0))
    const length = array.length
    const inputBuffer = mallocByteBuffer(length)
    inputBuffer.set(array)
    const counts = countOccurrences(inputBuffer.byteOffset, length, target.charCodeAt(0))
    free(inputBuffer)
    return counts
}

m.onRuntimeInitialized = () => {
    const numbersObj = swapNumbers({ first: 4, second: 5 })
    console.log(numbersObj)

    const plusNumbers = plusArrays([1, 2, 3, 4, 5])
    console.log(plusNumbers)

    const count = countOccurrences('WebAssembly', 's')
    console.log(`count=${count}`)
}
